library IEEE;
use  IEEE.STD_LOGIC_1164.all;
use  IEEE.STD_LOGIC_ARITH.all;
use  IEEE.STD_LOGIC_UNSIGNED.all;

-- Mquina de Estados Finitos (Modelo de Moore)

ENTITY moore IS
    PORT (clk,reset : IN BIT;
          sensor, botao: IN BIT;
          estado_out: OUT STD_LOGIC_VECTOR(7 DOWNTO 0); 
          sirene: OUT BIT);
END moore;

ARCHITECTURE behavior OF moore IS
 TYPE estados IS (desligado, ativado, disparado);
 signal estado: estados;
 
BEGIN
	PROCESS (clk)        
    
    BEGIN
		if (reset = '1') then
			estado <= desligado;
		
		--state transition logic
		elsif (clk'event) and (clk = '1') then    
			case estado is
				WHEN desligado =>
					if (botao = '1') then
						estado <= ativado;					
                    end if;
				WHEN ativado =>
					if (botao = '1') then
						estado <= desligado;					
                    elsif (sensor = '1') then
						estado <= disparado;					
                    end if;
				WHEN disparado => 
					if (botao = '1') then
						estado <= desligado;										
                    end if;                    
            end case;
		end if;
		    
    END PROCESS;
    
    PROCESS (reset,estado)
	BEGIN
		if (reset = '1') then
			sirene <= '0';
			estado_out <= x"00"; 		
		else
		
		--action logic
			case estado is
				WHEN desligado =>
					sirene <= '0';                    
				WHEN ativado =>
					sirene <= '0'; 
				WHEN disparado => 
					sirene <= '1';
				WHEN Others =>
					sirene <= '0'; 	                    
			end case;
				
		--monitor logic
			case estado is
				WHEN desligado =>
					estado_out <= x"00";                   
				WHEN ativado =>
					estado_out <= x"01";
				WHEN disparado => 
					estado_out <= x"02";
			end case;
		end if;
			
    END PROCESS;    
    
END behavior;	